@@ -5,27 +5,34 @@ All models defined here are abstract, which means no tables are created in
 the feincms\_ namespace.
 """
 
+from __future__ import absolute_import, unicode_literals
+
+try:
+    from collections import OrderedDict
+except ImportError:
+    from django.utils.datastructures import SortedDict as OrderedDict
+
+from functools import reduce
 import sys
 import operator
 import warnings
 
+import django
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ImproperlyConfigured
 from django.db import connections, models
 from django.db.models import Q
-from django.db.models.fields import FieldDoesNotExist
-from django.db.models.loading import get_model
 from django.forms.widgets import Media
-from django.template.loader import render_to_string
-from django.utils.datastructures import SortedDict
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text, python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from feincms import ensure_completely_loaded
+from feincms._internal import get_model
 from feincms.extensions import ExtensionsMixin
 from feincms.utils import copy_model_instance
 
 
+@python_2_unicode_compatible
 class Region(object):
     """
     This class represents a region inside a template. Example regions might be
@@ -38,8 +45,8 @@ class Region(object):
         self.inherited = args and args[0] == 'inherited' or False
         self._content_types = []
 
-    def __unicode__(self):
-        return force_unicode(self.title)
+    def __str__(self):
+        return force_text(self.title)
 
     @property
     def content_types(self):
@@ -48,17 +55,21 @@ class Region(object):
         of (content type key, beautified content type name) tuples
         """
 
-        return [(ct.__name__.lower(), ct._meta.verbose_name)
-                for ct in self._content_types]
+        return [
+            (ct.__name__.lower(), ct._meta.verbose_name)
+            for ct in self._content_types
+        ]
 
 
+@python_2_unicode_compatible
 class Template(object):
     """
     A template is a standard Django template which is used to render a
     CMS object, most commonly a page.
     """
 
-    def __init__(self, title, path, regions, key=None, preview_image=None):
+    def __init__(self, title, path, regions, key=None, preview_image=None,
+                 **kwargs):
         # The key is what will be stored in the database. If key is undefined
         # use the template path as fallback.
         if not key:
@@ -68,6 +79,10 @@ class Template(object):
         self.title = title
         self.path = path
         self.preview_image = preview_image
+        self.singleton = kwargs.get('singleton', False)
+        self.child_template = kwargs.get('child_template', None)
+        self.enforce_leaf = kwargs.get('enforce_leaf', False)
+        self.urlconf = kwargs.get('urlconf', None)
 
         def _make_region(data):
             if isinstance(data, Region):
@@ -77,8 +92,8 @@ class Template(object):
         self.regions = [_make_region(row) for row in regions]
         self.regions_dict = dict((r.key, r) for r in self.regions)
 
-    def __unicode__(self):
-        return force_unicode(self.title)
+    def __str__(self):
+        return force_text(self.title)
 
 
 class ContentProxy(object):
@@ -98,7 +113,7 @@ class ContentProxy(object):
         self.db = item._state.db
         self._cache = {
             'cts': {},
-            }
+        }
 
     def _inherit_from(self):
         """
@@ -155,7 +170,7 @@ class ContentProxy(object):
     def _fetch_content_type_count_helper(self, pk, regions=None):
         tmpl = [
             'SELECT %d AS ct_idx, region, COUNT(id) FROM %s WHERE parent_id=%s'
-            ]
+        ]
         args = []
 
         if regions:
@@ -164,10 +179,12 @@ class ContentProxy(object):
             args.extend(regions * len(self.item._feincms_content_types))
 
         tmpl.append('GROUP BY region')
-        tmpl = u' '.join(tmpl)
+        tmpl = ' '.join(tmpl)
 
-        sql = ' UNION '.join([tmpl % (idx, cls._meta.db_table, pk)\
-            for idx, cls in enumerate(self.item._feincms_content_types)])
+        sql = ' UNION '.join([
+            tmpl % (idx, cls._meta.db_table, pk)
+            for idx, cls in enumerate(self.item._feincms_content_types)
+        ])
         sql = 'SELECT * FROM ( ' + sql + ' ) AS ct ORDER BY ct_idx'
 
         cursor = connections[self.db].cursor()
@@ -180,7 +197,7 @@ class ContentProxy(object):
 
         return _c
 
-    def _popuplate_content_type_caches(self, types):
+    def _populate_content_type_caches(self, types):
         """
         Populate internal caches for all content types passed
         """
@@ -191,39 +208,51 @@ class ContentProxy(object):
                 counts_by_type.setdefault(
                     self.item._feincms_content_types[ct_idx],
                     [],
-                    ).append((region, pk))
+                ).append((region, pk))
 
         # Resolve abstract to concrete content types
-        content_types = (cls for cls in self.item._feincms_content_types
-                 if issubclass(cls, tuple(types)))
+        content_types = (
+            cls for cls in self.item._feincms_content_types
+            if issubclass(cls, tuple(types))
+        )
 
         for cls in content_types:
             counts = counts_by_type.get(cls)
             if cls not in self._cache['cts']:
                 if counts:
-                    self._cache['cts'][cls] = list(cls.get_queryset(
-                        reduce(operator.or_, (
-                            Q(region=r[0], parent=r[1]) for r in counts))))
+                    self._cache['cts'][cls] = list(cls.get_queryset(reduce(
+                        operator.or_,
+                        (Q(region=r[0], parent=r[1]) for r in counts)
+                    )))
                 else:
                     self._cache['cts'][cls] = []
 
+        # share this content proxy object between all content items
+        # so that each can use obj.parent.content to determine its
+        # relationship to its siblings, etc.
+        for cls, objects in self._cache['cts'].items():
+            for obj in objects:
+                setattr(obj.parent, '_content_proxy', self)
+
     def _fetch_regions(self):
         """
         Fetches all content types and group content types into regions
         """
 
         if 'regions' not in self._cache:
-            self._popuplate_content_type_caches(
+            self._populate_content_type_caches(
                 self.item._feincms_content_types)
             contents = {}
             for cls, content_list in self._cache['cts'].items():
                 for instance in content_list:
                     contents.setdefault(instance.region, []).append(instance)
 
-            self._cache['regions'] = dict((
-                region,
-                sorted(instances, key=lambda c: c.ordering),
-                ) for region, instances in contents.iteritems())
+            self._cache['regions'] = dict(
+                (
+                    region,
+                    sorted(instances, key=lambda c: c.ordering),
+                ) for region, instances in contents.items()
+            )
 
         return self._cache['regions']
 
@@ -241,7 +270,7 @@ class ContentProxy(object):
         content_list = []
         if not hasattr(type_or_tuple, '__iter__'):
             type_or_tuple = (type_or_tuple,)
-        self._popuplate_content_type_caches(type_or_tuple)
+        self._populate_content_type_caches(type_or_tuple)
 
         for type, contents in self._cache['cts'].items():
             if any(issubclass(type, t) for t in type_or_tuple):
@@ -352,7 +381,7 @@ def create_base_model(inherit_from=models.Model):
             """
 
             if not hasattr(cls, '_feincms_templates'):
-                cls._feincms_templates = SortedDict()
+                cls._feincms_templates = OrderedDict()
                 cls.TEMPLATES_CHOICES = []
 
             instances = cls._feincms_templates
@@ -364,12 +393,21 @@ def create_base_model(inherit_from=models.Model):
                 instances[template.key] = template
 
             try:
-                field = cls._meta.get_field_by_name('template_key')[0]
-            except (FieldDoesNotExist, IndexError):
-                cls.add_to_class('template_key',
-                    models.CharField(_('template'), max_length=255, choices=())
-                    )
-                field = cls._meta.get_field_by_name('template_key')[0]
+                field = next(iter(
+                    field for field in cls._meta.local_fields
+                    if field.name == 'template_key'))
+            except (StopIteration,):
+                cls.add_to_class(
+                    'template_key',
+                    models.CharField(_('template'), max_length=255, choices=(
+                        # Dummy choice to trick Django. Cannot be empty,
+                        # otherwise admin.E023 happens.
+                        ('__dummy', '__dummy'),
+                    ))
+                )
+                field = next(iter(
+                    field for field in cls._meta.local_fields
+                    if field.name == 'template_key'))
 
                 def _template(self):
                     ensure_completely_loaded()
@@ -380,15 +418,22 @@ def create_base_model(inherit_from=models.Model):
                         # return first template as a fallback if the template
                         # has changed in-between
                         return self._feincms_templates[
-                            self._feincms_templates.keys()[0]]
+                            list(self._feincms_templates.keys())[0]]
 
                 cls.template = property(_template)
 
-            cls.TEMPLATE_CHOICES = field._choices = [(
-                template.key,
-                template.title,
-                ) for template in cls._feincms_templates.values()]
-            field.default = field.choices[0][0]
+            cls.TEMPLATE_CHOICES = [
+                (template_.key, template_.title,)
+                for template_ in cls._feincms_templates.values()
+            ]
+            try:
+                # noqa https://github.com/django/django/commit/80e3444eca045799cc40e50c92609e852a299d38
+                # Django 1.9 uses this code
+                field.choices = cls.TEMPLATE_CHOICES
+            except AttributeError:
+                # Older versions of Django use that.
+                field._choices = cls.TEMPLATE_CHOICES
+            field.default = cls.TEMPLATE_CHOICES[0][0]
 
             # Build a set of all regions used anywhere
             cls._feincms_all_regions = set()
@@ -435,9 +480,17 @@ def create_base_model(inherit_from=models.Model):
                 app_label = cls._meta.app_label
                 ordering = ['ordering']
 
-            def __unicode__(self):
-                return u'%s on %s, ordering %s' % (
-                    self.region, self.parent, self.ordering)
+            def __str__(self):
+                return (
+                    '%s<pk=%s, parent=%s<pk=%s, %s>, region=%s,'
+                    ' ordering=%d>') % (
+                    self.__class__.__name__,
+                    self.pk,
+                    self.parent.__class__.__name__,
+                    self.parent.pk,
+                    self.parent,
+                    self.region,
+                    self.ordering)
 
             def render(self, **kwargs):
                 """
@@ -455,39 +508,6 @@ def create_base_model(inherit_from=models.Model):
 
                 raise NotImplementedError
 
-            def fe_render(self, **kwargs):
-                """
-                Frontend Editing enabled renderer
-                """
-
-                if 'request' in kwargs:
-                    request = kwargs['request']
-
-                    if (hasattr(request, 'COOKIES')
-                            and request.COOKIES.get('frontend_editing')):
-                        return render_to_string('admin/feincms/fe_box.html', {
-                            'content': self.render(**kwargs),
-                            'identifier': self.fe_identifier(),
-                            })
-
-                return self.render(**kwargs)
-
-            def fe_identifier(self):
-                """
-                Returns an identifier which is understood by the frontend
-                editing javascript code. (It is used to find the URL which
-                should be used to load the form for every given block of
-                content.)
-                """
-
-                return u'%s-%s-%s-%s-%s' % (
-                    cls._meta.app_label,
-                    cls._meta.module_name,
-                    self.__class__.__name__.lower(),
-                    self.parent_id,
-                    self.id,
-                    )
-
             def get_queryset(cls, filter_args):
                 return cls.objects.select_related().filter(filter_args)
 
@@ -499,16 +519,14 @@ def create_base_model(inherit_from=models.Model):
                 # from, therefore we ensure that the
                 # module is always known.
                 '__module__': cls.__module__,
-                '__unicode__': __unicode__,
+                '__str__': __str__,
                 'render': render,
-                'fe_render': fe_render,
-                'fe_identifier': fe_identifier,
                 'get_queryset': classmethod(get_queryset),
                 'Meta': Meta,
                 'parent': models.ForeignKey(cls, related_name='%(class)s_set'),
                 'region': models.CharField(max_length=255),
                 'ordering': models.IntegerField(_('ordering'), default=0),
-                }
+            }
 
             # create content base type and save reference on CMS class
 
@@ -519,10 +537,10 @@ def create_base_model(inherit_from=models.Model):
                     'FeinCMS auto-generates based on %s.%s. To avoid database'
                     'errors and import clashes, rename one of these classes.'
                     % (cls.__module__, name, cls.__module__, cls.__name__),
-                RuntimeWarning)
-
-            cls._feincms_content_model = type(name, (models.Model,), attrs)
+                    RuntimeWarning)
 
+            cls._feincms_content_model = python_2_unicode_compatible(
+                type(str(name), (models.Model,), attrs))
 
             # list of concrete content types
             cls._feincms_content_types = []
@@ -559,7 +577,7 @@ def create_base_model(inherit_from=models.Model):
 
         @classmethod
         def create_content_type(cls, model, regions=None, class_name=None,
-                **kwargs):
+                                **kwargs):
             """
             This is the method you'll use to create concrete content types.
 
@@ -621,21 +639,33 @@ def create_base_model(inherit_from=models.Model):
                 # everything ok
                 pass
 
-            # Next name clash test. Happens when the same content type is
-            # created for two Base subclasses living in the same Django
-            # application (github issues #73 and #150)
-            other_model = get_model(cls._meta.app_label, class_name)
-            if other_model:
-                warnings.warn(
-                    'It seems that the content type %s exists twice in %s.'
-                    ' Use the class_name argument to create_content_type to'
-                    ' avoid this error.' % (
-                        model.__name__,
-                        cls._meta.app_label),
-                    RuntimeWarning)
+            if django.VERSION < (1, 7):
+                # Next name clash test. Happens when the same content type is
+                # created for two Base subclasses living in the same Django
+                # application (github issues #73 and #150)
+                #
+                # FIXME This code does not work with Django 1.7, because
+                # get_model depends on the app cache which is not ready at
+                # this time yet.
+                try:
+                    other_model = get_model(cls._meta.app_label, class_name)
+                    if other_model is None:
+                        # Django 1.6 and earlier
+                        raise LookupError
+                except LookupError:
+                    pass
+                else:
+                    warnings.warn(
+                        'It seems that the content type %s exists twice in %s.'
+                        ' Use the class_name argument to create_content_type'
+                        ' to avoid this error.' % (
+                            model.__name__,
+                            cls._meta.app_label),
+                        RuntimeWarning)
 
             if not model._meta.abstract:
-                raise ImproperlyConfigured('Cannot create content type from'
+                raise ImproperlyConfigured(
+                    'Cannot create content type from'
                     ' non-abstract model (yet).')
 
             if not hasattr(cls, '_feincms_content_model'):
@@ -647,6 +677,7 @@ def create_base_model(inherit_from=models.Model):
                 db_table = '%s_%s' % (cls._meta.db_table, class_name.lower())
                 verbose_name = model._meta.verbose_name
                 verbose_name_plural = model._meta.verbose_name_plural
+                permissions = model._meta.permissions
 
             attrs = {
                 # put the concrete content type into the
@@ -658,12 +689,13 @@ def create_base_model(inherit_from=models.Model):
                 # the blog and the page module).
                 '__module__': cls.__module__,
                 'Meta': Meta,
-                }
+            }
 
             new_type = type(
-                class_name,
+                str(class_name),
                 (model, feincms_content_base,),
-                attrs)
+                attrs,
+            )
             cls._feincms_content_types.append(new_type)
 
             if hasattr(getattr(new_type, 'process', None), '__call__'):
@@ -673,8 +705,8 @@ def create_base_model(inherit_from=models.Model):
 
             # content types can be limited to a subset of regions
             if not regions:
-                regions = set([region.key for region
-                    in cls._feincms_all_regions])
+                regions = set([
+                    region.key for region in cls._feincms_all_regions])
 
             for region in cls._feincms_all_regions:
                 if region.key in regions:
@@ -753,7 +785,8 @@ def create_base_model(inherit_from=models.Model):
             # helper which can be used to ensure that either register_regions
             # or register_templates has been executed before proceeding
             if not hasattr(cls, 'template'):
-                raise ImproperlyConfigured('You need to register at least one'
+                raise ImproperlyConfigured(
+                    'You need to register at least one'
                     ' template or one region on %s.' % cls.__name__)
 
         @classmethod
@@ -762,9 +795,11 @@ def create_base_model(inherit_from=models.Model):
 
             # Check whether any content types have been created for this base
             # class
-            if (not hasattr(cls, '_feincms_content_types')
+            if (
+                    not hasattr(cls, '_feincms_content_types')
                     or not cls._feincms_content_types):
-                raise ImproperlyConfigured('You need to create at least one'
+                raise ImproperlyConfigured(
+                    'You need to create at least one'
                     ' content type for the %s model.' % cls.__name__)
 
         def copy_content_from(self, obj):
@@ -776,8 +811,8 @@ def create_base_model(inherit_from=models.Model):
 
             for cls in self._feincms_content_types:
                 for content in cls.objects.filter(parent=obj):
-                    new = copy_model_instance(content,
-                        exclude=('id', 'parent'))
+                    new = copy_model_instance(
+                        content, exclude=('id', 'parent'))
                     new.parent = self
                     new.save()
 
@@ -796,9 +831,12 @@ def create_base_model(inherit_from=models.Model):
         @classmethod
         def register_with_reversion(cls):
             try:
-                import reversion
+                from reversion import revisions as reversion
             except ImportError:
-                raise EnvironmentError("django-reversion is not installed")
+                try:
+                    import reversion
+                except ImportError:
+                    raise EnvironmentError("django-reversion is not installed")
 
             follow = []
             for content_type in cls._feincms_content_types:
@@ -808,5 +846,6 @@ def create_base_model(inherit_from=models.Model):
 
     return Base
 
+
 # Legacy support
 Base = create_base_model()
