@@ -1,3 +1,5 @@
+from __future__ import absolute_import, unicode_literals
+
 from django.shortcuts import render_to_response
 from django.template import RequestContext
 
@@ -12,5 +14,7 @@ def render_to_response_best_match(request, template_name, dictionary=None):
     dictionary = dictionary or {}
     dictionary['feincms_page'] = Page.objects.best_match_for_request(request)
 
-    return render_to_response(template_name, dictionary,
+    return render_to_response(
+        template_name,
+        dictionary,
         context_instance=RequestContext(request))
